#-----------------------------------------------------------------------------
# Name:        talker.py
# Purpose:     Driver for the Festival text-to-speech synthesis engine.
#              Uses the pexpect module to interactively drive Festival.
#
# Author:      Gary Cramblitt
#
# Created:     2003/31/08
# RCS-ID:      $Id: talker.py,v 1.1.1.1 2003/09/26 04:11:38 garycramblitt Exp $
# Copyright:   (c) 2003
# Licence:     GNU Public License (GPL)
# New field:   Whatever
#-----------------------------------------------------------------------------

import wx
import mypexpect
import string
import signal

class Talker:
    def __init__ (self):
        self.child = None
        self.talking = False
        self.logSayText = False
    
    def __del__(self):
        self.Close()

    def Open(self):
        self.child = mypexpect.spawn("festival --interactive")
        # Turn off pexpect's logging.
        self.child.setlog(None)
        # No need for echo.
        self.child.setecho(False)
        # Set flag to wait for festival prompt.
        self.talking = True
        

    def Close(self):
        if not self.child:
            if self.child.isalive():
                self.child.sendline("(quit)")
            self.child.close()
            del self.child
            self.child = NULL
            self.talking = False

    def CheckOpen(self):
        if not self.child:
            self.Open()
        # If festival crashed, restart it.
        if not self.child.isalive():
            self.Close()
            self.Open()

    def Say(self, text):
        self.CheckOpen()
        # Remove quotation characters.
        text = string.replace(text, "\\\"", "#!#!")
        text = string.replace(text, "\"", "\\\"")
        text = string.replace(text, "#!#!", "\\\"")
        # Remove comment lines.
        text = string.replace(text, "--", "")
        # Wait for festival prompt, if not already seen.
        self.WaitTilReady()
        # Send to festival.
        self.child.sendline("(SayText \"" + text + "\")")
        # Set flag to wait for festival prompt.
        self.talking = True
        if self.logSayText:
            wxLogMessage("(SayText \"" + text + "\")")

    def Ready(self):
        self.CheckOpen()
        if not self.talking:
            return True
        else:
            try:
                self.child.expect_exact("festival>", 0)
                self.talking = False
                return True
            except mypexpect.EOF:
                wxLogMessage("EOF")
                return False
            except mypexpect.TIMEOUT:
                return False
            except Exception:
                wxLogMessage("Exception")
                return False
            
    def Kill(self):
        if self.child:
            self.child.kill(signal.SIGKILL)

    def WaitTilReady(self):
        self.CheckOpen()
        if self.talking:
            self.child.expect_exact("festival>")
            self.talking = False
